# HG changeset patch
# User Elliot Thomas <elliot.thomas@bbc.co.uk>
# Date 1543836450 0
#      Mon Dec 03 11:27:30 2018 +0000
# Node ID 888e4c6836ef3ee87599c3042de9a0be990f3d65
# Parent  2117637f64e981e0e14c3a4b0509252fefd8a78a
Apply cache locking behaviour to stale cache entries.

Previously, this was only done for new cache entries.

diff -r 2117637f64e9 -r 888e4c6836ef src/http/ngx_http_cache.h
--- a/src/http/ngx_http_cache.h	Tue Nov 27 17:40:21 2018 +0300
+++ b/src/http/ngx_http_cache.h	Mon Dec 03 11:27:30 2018 +0000
@@ -188,6 +188,7 @@
 ngx_int_t ngx_http_file_cache_create(ngx_http_request_t *r);
 void ngx_http_file_cache_create_key(ngx_http_request_t *r);
 ngx_int_t ngx_http_file_cache_open(ngx_http_request_t *r);
+ngx_int_t ngx_http_file_cache_lock(ngx_http_request_t *r, ngx_http_cache_t *c);
 ngx_int_t ngx_http_file_cache_set_header(ngx_http_request_t *r, u_char *buf);
 void ngx_http_file_cache_update(ngx_http_request_t *r, ngx_temp_file_t *tf);
 void ngx_http_file_cache_update_header(ngx_http_request_t *r);
diff -r 2117637f64e9 -r 888e4c6836ef src/http/ngx_http_file_cache.c
--- a/src/http/ngx_http_file_cache.c	Tue Nov 27 17:40:21 2018 +0300
+++ b/src/http/ngx_http_file_cache.c	Mon Dec 03 11:27:30 2018 +0000
@@ -11,8 +11,6 @@
 #include <ngx_md5.h>
 
 
-static ngx_int_t ngx_http_file_cache_lock(ngx_http_request_t *r,
-    ngx_http_cache_t *c);
 static void ngx_http_file_cache_lock_wait_handler(ngx_event_t *ev);
 static void ngx_http_file_cache_lock_wait(ngx_http_request_t *r,
     ngx_http_cache_t *c);
@@ -399,7 +397,7 @@
 }
 
 
-static ngx_int_t
+ngx_int_t
 ngx_http_file_cache_lock(ngx_http_request_t *r, ngx_http_cache_t *c)
 {
     ngx_msec_t                 now, timer;
diff -r 2117637f64e9 -r 888e4c6836ef src/http/ngx_http_upstream.c
--- a/src/http/ngx_http_upstream.c	Tue Nov 27 17:40:21 2018 +0300
+++ b/src/http/ngx_http_upstream.c	Mon Dec 03 11:27:30 2018 +0000
@@ -923,6 +923,14 @@
         u->cache_status = NGX_HTTP_CACHE_HIT;
     }
 
+    if (rc == NGX_HTTP_CACHE_STALE)
+    {
+        rc = ngx_http_file_cache_lock(r, c);
+        if (rc == NGX_DECLINED) {
+            rc = NGX_HTTP_CACHE_STALE;
+        }
+    }
+
     switch (rc) {
 
     case NGX_OK:
